import numpy as N
import os.path as OP

def runemu(infile=None,outfile=None,params=N.array([0.1296,0.0224,0.97,-1.,0.8]),
           takeparamlogs=True,path=''):
    """
    CosmicEmuLog v.1.0 (c) 2011, Mark Neyrinck
    Mark.Neyrinck at gmail.com

    infile is read first, if it is set, and it exists.
    if outfile is set, output is to a file, else it is returned from the function

    The default params is the set of parameters for the fiducial Coyote
      Universe simulation 0

    if takeparamlogs=True, the input params are:     (omega_* = Omega_* h^2)
      omega_m, omega_b, n_s, w, sigma_8
    if takeparamlogs=False, the input params are the natural units of the emulator,
      ln(omega_m), ln(omega_b), n_s, w, ln(sigma_8^2)

    output: 
      k, P_delta, P_ln(1+delta), P_G(delta), P_(delta/sigma_cell)
      (sigma_cell is evaluated in cells of size 1300/256 Mpc ~ 4 Mpc/h).

    Warnings & Caveats:
    ------------------

    k is in 1/Mpc, not h/Mpc, and P(k)'s are in Mpc^3, not (Mpc/h)^3.
    If you want the little h's in the units (as you probably do), I
    suggest you run (the "real") CosmicEmu to get little h, which is
    one of the outputs, since it depends on the input parameters.  It is available at
    http://www.lanl.gov/projects/cosmology/CosmicEmu/emu.html

    The NGP pixel window function produces a substantial downturn at
    large k.  This program emulates the power spectra of a matter
    density field NGP-pixelated onto a grid with cells 1300/256 ~ 5
    Mpc in size, at z=0, not the infinite-resolution limit.  In fact,
    the infinite-resolution limit doesn't make sense for P_ln(1+delta)
    and P_G(delta)

    There are also artefacts from averaging together power spectra of
    HRS (Hamilton, Rimes & Scoccimarro 2007) sinusoidally-weighted
    density fields.  I used up to the 2nd overtone in each dimension,
    excluding over a factor of 4 in wavenumber at the low-k end.

    The weightings also artificially dampen the BAO wiggles, perhaps
    by a factor of 2.

    Also, k is not precisely equally spaced logarithmically; it was
    set by averaging mode wavenumbers in logarithmically spaced bins.

    For more information, see Neyrinck (2011), arXiv:1105.????, and
    the Coyote Universe papers; see 
    http://www.lanl.gov/projects/cosmology/CosmicEmu/pub.html

    """

    if not(OP.exists(path+'meanparams.txt') & OP.exists(path+'means.txt') &
           OP.exists(path+'dp.txt') & OP.exists(path+'dplog.txt') & 
           OP.exists(path+'dpg.txt') & OP.exists(path+'dpdos.txt')):
        print 'Some source files missing/not in specified directory.  Aborting.'
        return

    meanparams=N.loadtxt(path+'meanparams.txt')
    inputparams = 1.*params
    if infile != None:
        if OP.exists(infile):
            inputcosparam = N.loadtxt(infile)
        else:
            print 'Stated input file does not exist.  Using default paramters.'

    if takeparamlogs:
        inputparams[:2] = N.log(inputparams[:2])
        inputparams[-1] = 2.*N.log(inputparams[-1])

    print 'Input paramters: ln(omega_m), ln(omega_b), n_s, w, ln(sigma_8^2) ='
    print inputparams
    
    if ((inputparams[0] < N.log(0.12)) | (inputparams[0] > N.log(0.155)) | \
            (inputparams[1] < N.log(0.0215))|(inputparams[1] > N.log(0.0235))| \
            (inputparams[2] < 0.85)|(inputparams[2] > 1.05)|\
            (inputparams[3] < -1.3)|(inputparams[3] > -0.7)|\
            (inputparams[4] < 2.*N.log(0.61))|(inputparams[4] > 2.*N.log(0.9))):
        print 'Warning: at least one parameter outside range tested in the'
        print '   Coyote Universe suite'

    fluc = inputparams-meanparams

    #load geometric-mean power spectra among the simulations
    #warning: k is almost equally spaced in log, but not exactly.
    k,avgp,avgplog,avgpg,avgpdos=N.loadtxt(path+'means.txt',unpack=True)
    nk = len(k)

    #load derivatives wrt each cosmological parameter as a function of scale
    dp=N.loadtxt(path+'dp.txt')
    dplog=N.loadtxt(path+'dplog.txt')
    dpg=N.loadtxt(path+'dpg.txt')
    dpdos=N.loadtxt(path+'dpdos.txt')

    p = N.zeros(nk); plog=N.zeros(nk); pg = N.zeros(nk); pdos = N.zeros(nk)

    # Dot products of fluctuations in parameters with derivatives to get the fluctuated power spectra
    for ki in range(nk):
        p[ki]=avgp[ki]*N.exp(N.dot(fluc,dp[ki,:]))
        plog[ki]=avgplog[ki]*N.exp(N.dot(fluc,dplog[ki,:]))
        pg[ki]=avgpg[ki]*N.exp(N.dot(fluc,dpg[ki,:]))
        pdos[ki]=avgpdos[ki]*N.exp(N.dot(fluc,dpdos[ki,:]))

    if outfile == None:
        return k,p,plog,pg,pdos
    else:
        N.savetxt(outfile,N.transpose([k,p,plog,pg,pdos]),'%e')
        return

